/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "Configuration.h"
using namespace std;
/* initialize options parsing structs with getopt
 * - shortopts : struct contains short options tags where ":" means option require argument
 * - longopts : struct contains long options tags with ("name", haveArg?, NOT_USED, short_match)
 */
static char * shortopts = (char*)"ab:c:dfhj:k:m:no:p:r:st:vw:AB:C:D:FK:L:MNST:VY:"; // short options
static struct option longopts[]= {                                                // long options
	/* opt			has_arg			flag	val*/
	/**************************** GLOBAL *************************/
	{"help",		no_argument,		NULL,	'h'},
	{"master",		no_argument,		NULL,	'M'},
	{"slave",		no_argument,		NULL,	'S'},
	{"config-file",		required_argument,	NULL,	'c'},
	
	/****************************   JOB  *************************/
	{"silent",		no_argument,		NULL,	's'},
	{"verbose",		no_argument,		NULL,	'v'},
	{"very-verbose",	no_argument,		NULL,	'V'},
	{"keep-none",		no_argument,		NULL,	'N'},
	{"keep-fail",		no_argument,		NULL,	'n'},
	{"keep-success",	no_argument,		NULL,	'a'},
	{"keep-all",		no_argument,		NULL,	'A'},
	{"fake",		no_argument,		NULL,	'f'},
	{"no-fake",		no_argument,		NULL,	'F'},
	{"keep",		required_argument,	NULL,	'K'},
	{"verbosity",		required_argument,	NULL,	'Y'},
	{"maxt-job",		required_argument,	NULL,	't'},
	{"white",		required_argument,	NULL,	'w'},
	{"black",		required_argument,	NULL,	'b'},
		
	/**************************** SYSTEM *************************/
	{"output",		required_argument,	NULL,	'o'},
	{"autokill",		required_argument,	NULL,	'k'},
	{"build",		required_argument,	NULL,	'B'},
	{"nb-resources",	required_argument,	NULL,	'r'},
	{"nb-slaves",		required_argument,	NULL,	'j'},
	{"maxt-slave",		required_argument,	NULL,	'T'},
	{"mint-slave",		required_argument,	NULL,	'm'},
	{"launcher",		required_argument,	NULL,	'L'},
	{"compil-launcher",	required_argument,	NULL,	'C'},
	{"policy",		required_argument,	NULL,	'p'},
	{"size-flow",		required_argument,	NULL,	'D'},
	
	/*************************** END TAG *************************/
	{NULL,			no_argument,		NULL,	0}
};

Configuration::Configuration() : configStatus(EXEC_MASTER), configFile(NULL) {
}

Configuration::~Configuration() {
}

bool Configuration::isMaster() const {
	return configStatus == EXEC_MASTER;
}

void Configuration::printHelp() const {
	//printing banner and help menu for global configuration
	banner();
		
	//display global help
	cout 	<< COLOR_NRUN"         Usage: ./jchronoss [options] <XML Files> "COLOR_NORM"\n"
	
		<< COLOR_FAIL"\n GLOBAL OPTIONS :"COLOR_NORM"\n"
		<< PADDED_HELP << "   -h, --help " 		<< "Print this help.\n"
		<< PADDED_HELP << "   -M, --master "		<<"Launch JChronoss as master (recommended).\n"
		<< PADDED_HELP << "   -S, --slave " 		<<"Launch JChronoss as slave (rarely to use).\n"
		<< PADDED_HELP << "   -c, --config-file=<path> "<< "Specifies a configuration file to load. \n"
	;
	
	//display other helps
	configSystem.printHelp();
	configJobs.printHelp();
}

void Configuration::parseOptions ( int argc, char** argv ) {
	int option, optIndex;
	
	// pre actions
	assert(argc > 0);
	assert(argv != NULL);
	assert(argv[0] != NULL);
	
	// first arg = jchronoss path
	executablePathName = argv[0];
	
	// looking for a --config-file option (pre actions = loading configuration file)
	while((option = getopt_long(argc, argv, shortopts, longopts, &optIndex)) != -1){
		// if current option matches with config-file pattern
		if(option == 'c'){
			configFile = new string(optarg);
			assert(configFile != NULL);
			//init config file loading
			XMLConfigParser parser(*configFile);
			//parse jobs options from config file
			configJobs.loadFile(parser);
			//parse system options from config file
			configSystem.loadFile(parser);
			break;
		}
	}
	
	//re-init options parsing
	optind = 1;
	
	// override set configuration with command line options
	while((option = getopt_long(argc, argv, shortopts, longopts, &optIndex)) != -1){
		switch(option){
			case 'h':
				printHelp();
				exit(0);
				break;
			case 'M':
				configStatus = EXEC_MASTER;
				break;
			case 'S':
				configStatus = EXEC_SLAVE;
				break;
			case '?':
				printError("Some issues about options. See --help for further information !");
				exit(1);
				break;
			default:
				// if options is not considered as global configuration, looking for options in subConfiguration
				if(!configJobs.parseJobOptions(option)) //if not a job option
					configSystem.parseSystemOptions(option);
				break;
		}
	}

	// parsing arguments left which are not options (i.e. xml jobs files)
	if(optind != argc){
		while(optind != argc){
			configJobs.addJobsFiles(argv[optind++]);
		}
	}
	// some validity checks on configuration state
	if(isMaster()){
		banner();
		// checks options
		checkOptionsValidity();
	}
}

const JobConfiguration& Configuration::job() const {
	return configJobs;
}

const SystemConfiguration& Configuration::system() const {
	return configSystem;
}

string Configuration::getExeName() const {
	return executablePathName;
}
void Configuration::printConfiguration() const
{	
	//global configuration printing
	if(configFile != NULL){
		cout << COLOR_NRUN"+--------------------- GLOBAL CONFIGURATION -------------------+"COLOR_NORM << endl;
		cout << PADDED_OPT << "| - Configuration file : " << *configFile << endl;
	}
	
	//other printing
	cout << COLOR_NRUN"+--------------------- SYSTEM CONFIGURATION -------------------+"COLOR_NORM << endl;
	configSystem.printConfiguration();
	cout << COLOR_NRUN"+----------------------- JOB CONFIGURATION --------------------+"COLOR_NORM << endl;
	configJobs.printConfiguration();
	cout << COLOR_NRUN"+--------------------------------------------------------------+"COLOR_NORM << endl;
}

string* Configuration::getConfigFile() const
{
	return configFile;
}

void Configuration::checkOptionsValidity() const {

	//if resources number is not defined, unable to guess how to schedule.
	//Maybe this is required only if not default policy and execute sequentially in this case
	if(configSystem.getNbMaxResources() < 1)
		printError("Number of resources not set. Unable to schedule without that parameter !!\n ERROR        : Try to use --nb-resources | -r options or the configuration file.");
	
	//max slaves is always positive (size_t) so warning if user give < 0
	//if(configSystem.getNbMaxSlaves() == 0)
	//	printError("Number of slaves have to be greater than 0 !\n ERROR        : Try to use --nb-slaves | -j options or the configuration file.");
	
	//check intermediate wrappers exist
	if(!FileManager::isCreated(configSystem.getCompilationJobsLauncherCommand()))
		printError("The given Compilation launcher script not found");
		
	if(!FileManager::isCreated(configSystem.getJobsLauncherCommand()))
		printError("The given launcher script not found");
	
	//specific check on policy TIME
	if(configSystem.getPolicy() != POLICY_DEFAULT){
		//user should set max time for an allocation, to set upper bound for allocation
		if(configSystem.getMaxSlaveTime() == SystemConfiguration::DEFAULT_MAX_SLAVE_TIME)
			printWarning("You choose a policy where --maxt-slave should be defined. A bound limit should be set (--maxt-slave)");
		// user should set job time, used to compute job bloc size in allocation area.
		if(configJobs.getMaxJobTime() == JobConfiguration::DEFAULT_MAX_JOB_TIME)
			printWarning("Max Time for job not set !! (ignore this if you've specified job time individually)");
	}
	
	//min < max check
	if(configSystem.getMaxSlaveTime() < configSystem.getMinSlaveTime())
		printError("The max time allowed to a slave cannot be lower than the min one.\n ERROR        : See --maxt-slave and --mint-slave options");
	
	if(configSystem.getPolicy() < POLICY_DEFAULT || configSystem.getPolicy() > POLICY_CUSTOM){
		printError("Invalid policy. Choose in {0, 1, 2, 3} (see --help)");
	}

	if(!(*configSystem.getOutputDirectory() == "") && !FileManager::isCreated(configSystem.getOutputDirectory()))
		printError("Output path doesn't exist !");
	if(!FileManager::isCreated(configSystem.getBuildDirectory()))
		printError("Build path doesn't exist !");
}

