/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include "utils.h"
#include "FileManager.h"
#include "ConfigurationSystem.h"
#include "ConfigurationJob.h"

/// Enum describing modes as JCHRONOSS can be launched
typedef enum eExecMode{
	EXEC_MASTER = 0, ///< current execution is the master one
	EXEC_SLAVE= 1    ///< current execution is the slave one
} execMode; 

/// Object Configuration represents the whole configuration for JCHRONOSS
/**
 * this global configuration contains all information about jobs and system
 * configuration (including paths, contents, xml files, scheduling policies)
 * needed by the application. It's in this configuaration that type of execution
 * is setted (MASTER or SLAVE)
 */
class Configuration {
private:
	/************** MEMBERS **************/
	std::string executablePathName;    ///< path name of the current application (needed for recursive call)
	execMode configStatus;             ///< execution status (MASTER or SLAVE)
	SystemConfiguration configSystem;  ///< referent system configuration
	JobConfiguration configJobs;       ///< referent jobs configuration
	std::string* configFile;           ///< used to load file configuration instead of using options
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// standard Configuration constructor
	/** Its aims is to configure Configuration object and job/system configuration */
	explicit Configuration();
	
	/// Look for a matching between its options and the given chain
	/** 
	 * When JCHRONOSS parses options from command line, it looks for a match 
	 * between its options and the current one. If there is a matching, option 
	 * is setted and looking for is stopped.
	 * @param[in] arg the given option from the command line
	 * @return <b>true</b> if option matches
	 * @return <b>false</b> otherwise.
	 */
	bool parseGlobalOptions ( char* arg );

	/// Global function to parse the whole command line
	/**
	 * Then program starts, command line's arguments are provided to set the global 
	 * configuration (job+system)
	 * @param[in] argc number of arguments in command line ( = argc from main())
	 * @param[in] argv string array containing command line's arguments (= argv from main())
	 */
	void parseOptions(int argc, char ** argv);

	/// virtual destructor to unset current configuration
	virtual ~Configuration();
	
	/****** CONST ******/
	/// check if current processus is the master one
	/**
	 * Identifies the current application as the master processus. Used to differentiate
	 * master execution from slave one
	 * @return <b>true</b> if the processus is the master
	 * @return <b>false</b> otherwise
	 */
	bool isMaster() const;
	/// print JCHRONOSS help (--help | -h options)
	void printHelp() const;
	/// print current running configuration 
	/**
	 * This function should be called only in master application (avoiding useless printing)
	 */
	void printConfiguration() const;
	/// get Current JobConfiguration object with its own specifications
	/**
	 * Getting JobConfiguration from global configuration. This configuration, once setted,
	 * is read only. This is why we return a const reference of JobConfiguration.
	 * @return The current JobConfiguration reference.
	 */
	const JobConfiguration& job() const;
	/// get Current SystemConfiguration object with its own specifications
	/**
	 * Getting SystemConfiguration from global configuration. This configuration, once setted,
	 * is read only. This is why we return a const reference of SystemConfiguration.
	 * @return The current SystemConfiguration reference.
	 */
	const SystemConfiguration& system() const;
	/// exeName getter
	/**
	 * Allow to get path of himself in order to launch himself as a slave later
	 * @return A string containing JCHRONOSS binary file path
	 */
	std::string getExeName() const;
	/// get configuration file
	/**
	 * \return a FileManager pointer on current configuration file
	 * \return <b>NULL</b> if no file are found
	 */
	std::string * getConfigFile() const;
	/// check options coherency
	/**
	 * Check if run would be safe, according to options given by user 
	 */
	void checkOptionsValidity() const;
};

#endif // CONFIGURATION_H
