/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef JOBCONFIGURATION_H
#define JOBCONFIGURATION_H

#include "utils.h"
#include "FileManager.h"
#include "FilterBox.h"
#include "XMLConfigParser.h"
#include "Job.h"

/// Enum defining different verbose levels, i.e. screen displaying
typedef enum eVerboseMode{ 
	VERBOSE_SILENT=0,     ///< No jobs output (except jobs status line)
	VERBOSE_ONLY_ERROR=1, ///< print jobs output from failed jobs
	VERBOSE_ALL=2         ///< Print everything from every jobs
} verboseMode;
/// Enum defining different logging levels, i.e. output jobs kept in file (NOTHING, just failed, just success or everything)
typedef enum eLogIntoFileMode{
	LOG_NONE=0,         ///< keep nothing in JUnit files
	LOG_ONLY_FAILED=1,  ///< keep output from only failed jobs
	LOG_ONLY_SUCCESS=2, ///< keep output from only success jobs
	LOG_ALL=3          ///< keep output whatever jobs state
}logIntoFileMode;

/// Class defining a JobConfiguration, containing the whole information about jobs control
/**
 * A job configuration is a part of the global configuration.
 * In this object, we can found all options and configuration setted 
 * by default or by user and which are about jobs manipulation 
 * (execution, timers, filters, xml files, etc...)
 */
class JobConfiguration {
private:
	/************** MEMBERS **************/
	verboseMode verbosity;                            ///< application verbosity level
	logIntoFileMode logLevel;                         ///< "output conservation in file" level
	bool fakeExecution;                               ///< Defines if jobs have to be simulated (random times --> jobs won't be launched)
	unsigned int maxJobTime;                          ///< Defines the global max time. A job execution can't overflow this max time
	unsigned int nbMaxJobs;                           ///< Represents the max number of jobs to launch in a worker
	std::vector<FileManager*> jobsFilesList;          ///< Represents jobs XML lists
	std::vector<FileManager*> filtersWhiteFileslist;  ///< Represents jobs which have to be kept for the current running
	std::vector<FileManager*> filtersBlackFileslist;  ///< Represents jobs which mustn't be launched in the current execution
	
	
public:
	/************** STATICS **************/
	/****** CONST ******/
	const static size_t DEFAULT_MAX_JOB_TIME = Job::DEFAULT_MAX_JOB_TIME; ///< if no value specified by user, the global max time for a job is this static constant
	const static unsigned int DEFAULT_MAX_NB_JOBS = 0;                    ///< max number of jobs which can be stored in a worker (if policy used it)
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// standard JobConfiguration constructor
	/**
	 * Its aim is to configure JobConfiguration object after a call to
	 * global Configuration object initialization.
	 */
	explicit JobConfiguration();
	///Look for a matching option in its own pattern
	/**
	 * When JCHRONOSS parses options from command line, it looks for a match 
	 * between its options and the current one. If there is a matching, option 
	 * is setted and looking for is stopped and class memer is setted.
	 * \param[in] option the current option from the command line (by getopt)
	 * \return <b>true</b> if an option matches
	 * \return <b>false</b> otherwise
	 */
	bool parseJobOptions(int option);
	///add list of files in string to the current list
	/**
	 * When arguments are parsed, it's possible to provide a string as a list
	 * of files (file1,file2,file3...). This function split each list item and
	 * add it to the given list (generally the list is a class member but as
	 * we want to use one function prototype for different lists, we must 
	 * provide the list reference in parameter)
	 * \param[in] chain The chain containing a comma-separated list
	 * \param[out] list The list were item will be inserted
	 */
	void addFiles(std::string* chain, std::vector<FileManager*>& list);
	///add current argument (a XML file) to the list of jobs XML files list
	/**
	 * if an argument doesn't match with options (neither system, global nor job)
	 * this argument is considered as a job XML input file. This function allows
	 * to add this file to the current list.
	 * \param argv The current chain containing the file path
	 */
	void addJobsFiles ( char* argv );
	///get XML files list (used only if list need to be updated
	/** 
	 * There are no const on this function. Thus, the list reference can be
	 * used to update XML files list
	 * \return A reference on XML files list
	 */
	std::vector<FileManager*>& getJobsFilesNoLock();
	/// load configuration file options for current job configuration
	/**
	 * Use a XMLConfigParser to get config options from file
	 * \param[in] parser XMLConfigParser initialized by parent
	 */
	void loadFile(const XMLConfigParser & parser);
	///virtual destructor to unset current JobConfiguration
	virtual ~JobConfiguration();	
	
	/****** CONST ******/
	/// print help menu for current JobConfiguration
	void printHelp () const;
	/// print current JobConfiguration on screen
	void printConfiguration() const;
	/// get verbosity level set by the configuration
	/**
	 * \return verbose value
	 */
	verboseMode getVerbosity() const; 
	/// get log level set by the configuration
	/**
	 * \return loggin level value
	 */
	logIntoFileMode getLogLevel() const;
	/// Check if fake mode is enabled
	/** 
	 * if fake Execution is activated, no jobs will be launched and execution
	 * time will be random-generated (used for scheduling policy tests)
	 * \return <b>True</b> if fake mode is enabled
	 * \return <b>False</b> otherwise
	 */
	bool isFake() const;
	/// Get the current max job time alloweb by the configuration
	/** 
	 * \return The time value
	 */
	unsigned int getMaxJobTime() const;
	///Get Jobs XML files list (const method). 
	/**
	 * If updated are required, you can see getJobsFilesNoLock
	 * \see getJobsFilesNoLock 
	 * \return a const reference on jobs list (unmodifiable) 
	 */
	const std::vector<FileManager*>& getJobsFiles() const;
	/// get XML files list containing jobs which have to be launched
	/**
	 * \return A const reference on the list (unmodifiable)
	 */
	const std::vector<FileManager*>& getFiltersWhiteFiles() const;
	/// get XML files list containing jobs which mustn't be executed
	/**
	 * \return a const reference on the list (unmodifiable)
	 */
	const std::vector<FileManager*>& getFiltersBlackFiles() const;
	/// get the max number of jobs allowed
	/**
	 * \return the value of max number allowed
	 * \return DEFAULT_MAX_NB_JOBS by default
	 */
	unsigned int getNbMaxJobs() const;
};

#endif // JOBCONFIGURATION_H
