/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef FILTER_H
#define FILTER_H

#include "utils.h"
#include "FilterRule.h"

/// Enum of the filter entity mode
typedef enum eFilterMode {
	WHITELIST = 0, ///< current filter is considered as a whitelist
	BLACKLIST = 1, ///< current filter is considered as a blacklist
	WHATEVER = 3   ///< current filter have no effect (default)
} FilterMode;

/// Represents the entity group applied to jobs selection
/**
 * A filter is constituted with Filter items list (FilterRule) and a mode
 * applied to this list. The filter can be inclusive, exclusive or with no
 * effect. This list is generated from an input file given by confguration
 */
class Filter {
private:
	/************** MEMBERS **************/
	FilterMode type;                ///< mode to apply to current filter
	std::vector<FilterRule*> rules; ///< list of rules to apply
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	Filter();
	/// constructor by copy
	Filter ( const Filter& other );
	/// add a Filter item to the filter
	/**
	 * Help to construct the filter.
	 * \param[in] rule the new rule to insert
	 */
	void addRule(FilterRule* rule);
	///load filter items from a given file
	/**
	 * insert each line of the file as a rule in the current filter.
	 * Alos, apply the mode to fhe filter
	 * \param[in] fileName the file path used as a filter
	 * \param[in] mode The mode to apply to the filter
	 * \return <b>False</b> if file cannot be opened
	 * \return <b>True</b> otherwise
	 */
	bool fillFromFile(std::string fileName, FilterMode mode);
	/// virtual destructor to unset a filter
	virtual ~Filter();
	
	/****** CONST ******/
	///check if given job is valid according to the current filter
	/**
	 * We iterate on each rule and check if a rule matches with given job.
	 * According to mode, if it's whitelist, the job is kept, otherwise it's
	 * rejected.
	 * \param[in] name the job name to match with filter
	 * \return <b>True</b> if job need to be kept
	 * \return <b>False</b> otherwise
	 */
	bool accept(std::string name) const;
	/// display the current filter (used to debug)
	void display() const;
};

#endif // FILTER_H
