/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "Job.h"
using namespace std;

size_t Job::id = 0;

Job::Job() : status(NOT_RUN) {}

Job::Job ( std::string name, std::string command, std::string packageName, size_t nbRes,  int rc, double time, std::vector< std::string* > deps, std::vector< JobConstraint* > constraints, FileManager* file ) :
	name(name), command(command), packagePath(packageName),  nbDeps(deps.size()), nbTries(0), nbResources(nbRes), expectedReturnCode(rc), timer(time), vDepsNamesTab(deps), vConstraints(constraints), status(NOT_RUN), result(NULL), referentFile(file)
{
	myId = ++id;
}

Job::~Job() {
	if(result != NULL) delete result;
	for(vector<string*>::iterator it = vDepsNamesTab.begin(); it != vDepsNamesTab.end();){
		delete *it;
		it = vDepsNamesTab.erase(it);
	}
	
	for(vector<JobConstraint*>::iterator it = vConstraints.begin(); it != vConstraints.end();){
		delete (*it);
		it = vConstraints.erase(it);
	}

}

bool Job::operator== ( const Job& other ) const {
	return (name == other.name && command == other.command && packagePath == other.packagePath);
}

void Job::display() const {
	cout << "|\t - "<< name << " --> " << hex << this << dec
		<< "\n|\t\t - command: \"" << command << "\""
		<< "\n|\t\t - Expected code: " << expectedReturnCode
		<< "\n|\t\t - status: " << status
		<< "\n|\t\t - NbDeps: " << nbDeps;
	for(size_t i=0; i<nbDeps; i++)
		cout<< "\n|\t\t\t --> " << *vDepsNamesTab.at(i);
	
	cout << "\n|\t\t - nbConstraints: " << vConstraints.size();
	for(size_t i=0; i<vConstraints.size(); i++)
		cout<< "\n|\t\t\t -->" << vConstraints.at(i);
	cout << "\n|\t\t - Into " << referentFile->toString();
	cout << endl;
	if(result != NULL)
		result->display();
	
	for(vector<JobConstraint*>::const_iterator it = vConstraints.begin(); it != vConstraints.end(); it++)
		(*it)->display();

}

bool Job::isValid ( JobConstraint constraint ) const {
	
	if(constraint.isBelongingTo("")) return true;
	
	for(size_t i=0; i < vConstraints.size(); i++)
		if(constraint == *(vConstraints.at(i))) return true;
		
	return false;
}

void Job::addConstraint ( JobConstraint* constraint ) {
	assert(constraint != NULL);
	vConstraints.push_back(constraint);
}

void Job::addDependency ( Job* dep ) {
	vDeps.push_back(dep);
}

const std::vector< JobConstraint* >& Job::getConstraints() const {
	return vConstraints;
}

const std::vector< std::string* >& Job::getDepsNames() const {
	return vDepsNamesTab;
}

const std::vector< Job* >& Job::getDeps() const {
	return vDeps;
}

size_t Job::getNbResources() const {
	return nbResources;
}

int Job::getNbDeps() const {
	return nbDeps;
}
size_t Job::getId() const
{
	return myId;
}

string Job::getName() const {
	return name;
}

void Job::updateStatus ( JobStatus status ) {
	this->status = status;
}

JobStatus Job::getStatus() const {
	return status;
}

void Job::addResult(int rc, double time, double begin, std::string data){
	result = new JobResult;
	result->fillHeader(name, myId, rc, time, begin);
	result->insertData(data);
}

std::string Job::getCommand() const
{
	return command;
}
int Job::getExpectedReturn() const
{
	return expectedReturnCode;
}

 const JobResult& Job::getResult() const
{
	return *result;
}

FileManager* Job::getReferentFile() const
{
	assert(referentFile != NULL);
	return referentFile;
}

string Job::getPackage() const
{
	return packagePath;
}

void Job::setResult(JobResult* res)
{
	assert(res != NULL);
	result = res;
}

void Job::setId(size_t value)
{
	myId = value;
}

size_t Job::getNbTries() const
{
	return nbTries;
}
double Job::getExpectedTime() const
{
	return timer;
}

void Job::setExpectedTime(double time){
	timer = time;
}

bool Job::isDepInvalid() const
{
	bool invalid = false;
	vector<Job*>::const_iterator it = vDeps.begin();
	while(!invalid && it != vDeps.end()){
		if((*it)->getStatus() == FAILED || (*it)->getStatus() == NOT_RUNNABLE || (*it)->getStatus() == INVALID_DEPS || (*it)->getStatus() == MUCH_TRIES) invalid = true;
		it++;
	}
	return invalid;
}

