/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#include "JobManager.h"

using namespace std;

JobManager::JobManager() {

}

JobManager::JobManager(Configuration* config) {
	assert(config != NULL);
	jobsList = NULL;
	nbRemainingJobs = 0;
	nbJobs = 0;
	nbLists = config->system().getNbMaxResources();
	this->config = config;
}


JobManager::~JobManager() {
	
	for(list<Job*>::iterator it = executedJobsList.begin(); it != executedJobsList.end();){
		delete (*it);
		it = executedJobsList.erase(it);
	}
	
	delete [] jobsList;
}

void JobManager::addExecutedJob ( Job * current ) {
	assert(current != NULL);
	executedJobsList.push_back(current);
}

void JobManager::addJob ( Job * current ) {
	assert(current != NULL);
	jobsList[current->getNbResources() - 1].push_back(current);
}
int JobManager::getHigherValidList(size_t nbResources) const {
	int val = -1;
	size_t size = (nbLists < nbResources) ? nbLists : nbResources; 
	for(int i = size -1 ; i >= 0; i--){
		if(!jobsList[i].empty()){
			val = i;
			break;
		}
	}
	return val;

}

size_t JobManager::getNbJobs() const
{
  return nbJobs;
}


void JobManager::cleanInvalidJobs() {
	string stringTries = 
		"/!\\ TOO MUCH TRIES TO LAUNCH; This test is considered as failed !!! /!\\";
	string stringNotRunnable = 
		"/!\\ NOT RUNNABLE TEST; Some dependencies aren't resolved. This test cannot be launched !!! /!\\";
	string stringInvalid = 
		"/!\\ INVALID DEPS; Some dependencies wasn't found in our list for this test. By safety, this test wasn't be scheduled  !!! /!\\";

	for(size_t i = 0; i < nbLists; i++){
		for(list<Job*>::iterator it = jobsList[i].begin(); it != jobsList[i].end();){
			if((*it)->isDepInvalid()){
				(*it)->updateStatus(NOT_RUNNABLE);
			} else if((*it)->getNbTries() > Job::DEFAULT_MAX_TRIES)
				(*it)->updateStatus(MUCH_TRIES);
			switch((*it)->getStatus()){
				case MUCH_TRIES:
					addExecutedJob((*it));
					nbRemainingJobs--;
					printLine(PREFIX_URUN, "     -/-      (------) %8.2f   %s\n",0.00, (*it)->getName().c_str());
					(*it)->addResult(-1, 0.0, 0.0, stringTries);
					it = jobsList[i].erase(it);
					break;
				
				case INVALID_DEPS:
					addExecutedJob((*it));
					nbRemainingJobs--;
					printLine(PREFIX_URUN, "     -/-      (------) %8.2f   %s\n",0.00, (*it)->getName().c_str());
					(*it)->addResult(-1, 0.0, 0.0, stringInvalid);
					it = jobsList[i].erase(it);
					break;
				
				case NOT_RUNNABLE:
					addExecutedJob((*it));
					nbRemainingJobs--;
					printLine(PREFIX_URUN, "     -/-      (------) %8.2f   %s\n",0.00, (*it)->getName().c_str());
					(*it)->addResult(-1, 0.0, 0.0, stringNotRunnable);
					it = jobsList[i].erase(it);
					break;
				
				default: 
					it++; 
					break;
			}
		}
	}
}

bool JobManager::currentListIsEmpty ( int indice ) const {
	return jobsList[indice].empty();
}
void JobManager::display() const {
	cout 	<< "+---------------- JOB MANAGER -----------------+" << endl
		<< "| - Tests = " << nbJobs << endl
		<< "| - Remaining = " << nbRemainingJobs << endl
		<< "| - Nb lists = " << nbLists << endl
		<< "+------------------- TO RUN -------------------+" << endl;

	for(size_t i = 0 ; i < nbLists; i++){
		if(!jobsList[i].empty()){
			cout	<< "|- Required : " << i+1 << endl;
			for(list<Job*>::iterator it = jobsList[i].begin(); it != jobsList[i].end(); it++){
				//cout << "|\t - " << (*it)->getName() << endl;
				(*it)->display();
			}
		}
	}
	cout << "+-------------------EXECUTED ------------------+" << endl;
	for(list<Job*>::const_iterator it = executedJobsList.begin(); it != executedJobsList.end(); it++){
		(*it)->display();
	}
			
	cout << "+----------------------------------------------+" << endl;
	

}
bool JobManager::isEmpty() const {
	for(size_t i = 0 ; i < nbLists; i++)
		if(!jobsList[i].empty()) return false;
	return true;
}

void JobManager::pullJobsFromFiles () {
	int cpt = 0;
	size_t val = 0, max_tmp = 0, max = 0;
	FilterBox filterbox;
	vector<FileManager*> vFiles = config->job().getJobsFiles();
	list<Job*>* *tabJobs = new list<Job*>*[vFiles.size()];

	for(vector<FileManager*>::const_iterator white = config->job().getFiltersWhiteFiles().begin(); white != config->job().getFiltersWhiteFiles().end(); white++){
		filterbox.addFilter((*white), WHITELIST);
	}
	
	for(vector<FileManager*>::const_iterator black = config->job().getFiltersBlackFiles().begin(); black != config->job().getFiltersBlackFiles().end(); black++){
		filterbox.addFilter((*black), BLACKLIST);
	}

#ifdef ENABLE_OPENMP
	#pragma omp parallel shared(max) firstprivate(max_tmp)
	{
		#pragma omp for nowait
#endif
			for(size_t i = 0; i < vFiles.size(); i++){
				XMLJobParser parser(vFiles[i], &filterbox);
				tabJobs[i] = parser.parseFromFile(&val);
				if(tabJobs[i] == NULL) continue;
				for(list<Job*>::iterator it = tabJobs[i]->begin(); it != tabJobs[i]->end(); it++){
					max_tmp = std::max(max_tmp, (*it)->getNbResources());
				}
			}
#ifdef ENABLE_OPENMP
		#pragma omp critical
		{
#endif
			max = std::max(max, max_tmp);
#ifdef ENABLE_OPENMP
		}
	}
#endif
	XMLParser::cleanUp();

	nbLists = max;
	if(nbLists > config->system().getNbMaxResources())
		printError("Some jobs need more resources than the max availables resources !!!!");
	jobsList = new list<Job*>[nbLists];
	
	for(size_t i = 0; i < vFiles.size(); i++){
		if(tabJobs[i] == NULL) continue;
		for(list<Job*>::iterator it = tabJobs[i]->begin(); it != tabJobs[i]->end();){
			if((*it)->getExpectedTime() == -1)
				(*it)->setExpectedTime(config->job().getMaxJobTime());
			addJob(*it);
			cpt++;
			it = tabJobs[i]->erase(it);
		}
		delete tabJobs[i];
	}
	delete [] tabJobs;
	
	if(config->isMaster())
		nbRemainingJobs = cpt;
	else
		nbRemainingJobs = val;
	nbJobs = cpt;
	resolveJobsDeps();
}

void JobManager::pushInOneFile(FileManager* file){
	
	string notRun = "/!\\ For unknown reasons, this test hadn't be launched. It is considered as an error !!! /!\\";
	string curPathName, outputFileName, command, data, name, group;
	ostringstream mainContentString, headerString;
	FileManager* output = NULL;
	size_t nbFailures = 0, nbSuccess = 0, nbErrors = 0, nbSkipped = 0;
	double totalTime = 0.0;
	
	assert(file != NULL);
	outputFileName = (*config->system().getOutputDirectory() == "") ?  file->getPath()+DEFAULT_OUTPUT_FILENAME+file->getBaseName() : *config->system().getOutputDirectory()+DEFAULT_OUTPUT_FILENAME+file->getBaseName();
	output =  new FileManager(&outputFileName, OUTPUT_ACCESS);
	
	CHECK(output->open());
	
	//time display control
	mainContentString << fixed << setprecision(2);
	headerString << fixed << setprecision(2);
	
	for(list<Job*>::iterator it = executedJobsList.begin(); it != executedJobsList.end(); it++){
		if((*it)->getReferentFile() != file) continue;
		
		group = (*it)->getPackage();
		name = (*it)->getName();
		command = (*it)->getCommand();
		data = (*it)->getResult().getData();
		
		HTMLEncoding(group);
		HTMLEncoding(name);
		HTMLEncoding(command);
		HTMLEncoding(data);
	
		totalTime += (*it)->getResult().getTime();
		
		mainContentString << "\t<testcase classname=\"" << group <<"\" name=\""<< name << "\" time=\"" << (*it)->getResult().getTime() << "\" >\n";
				
		switch((*it)->getStatus()){
			case PASSED:	
				mainContentString << "\t\t<success message=\"" << command << "\" />" << endl;
				if(config->job().getLogLevel() == LOG_ALL || config->job().getLogLevel() == LOG_ONLY_SUCCESS)
					mainContentString << "\t\t<system-out>"<< data << endl <<"\t\t</system-out>" << endl;
				
				nbSuccess++;
				break;
			case FAILED:
				mainContentString << "\t\t<failure message=\"" << command << "\" type=\"ExecutionError\"/>" << endl;
				
				if(config->job().getLogLevel() == LOG_ALL || config->job().getLogLevel() == LOG_ONLY_FAILED){
					mainContentString << "\t\t<system-out>"<< data << endl << "\t\t</system-out>" << endl;
				}
				nbFailures++;
				break;
			case NOT_RUN:
			case MUCH_TRIES:
				mainContentString << "\t\t<skipped message=\""<< command << "\" />" << endl;
				if(config->job().getLogLevel() == LOG_ALL || config->job().getLogLevel() == LOG_ONLY_FAILED)
					mainContentString << "\t\t<system-out>"<< data << endl << "\t\t</system-out>" << endl;
				nbSkipped++;
				break;
			case NOT_RUNNABLE:
				mainContentString << "\t\t<skipped message=\""<< command << "\" />" << endl;
				if(config->job().getLogLevel() == LOG_ALL || config->job().getLogLevel() == LOG_ONLY_FAILED)
					mainContentString << "\t\t<system-out>"<< data << endl << "\t\t</system-out>" << endl;
				nbSkipped++;
				break;
			case INVALID_DEPS:
				mainContentString << "\t\t<error message=\"" << command << "\" type=\"EnvironmentError\"/>" << endl;
				
				if(config->job().getLogLevel() == LOG_ALL || config->job().getLogLevel() == LOG_ONLY_FAILED)
					mainContentString << "\t\t<system-out>" << data << endl << "</system-out>" << endl;
				
				nbErrors++;
				break;
		}
		      mainContentString << "\t</testcase>" << endl;

	}

	// if empty file, avoid to create it
	if(mainContentString.str().size() == 0){
		CHECK(output->close());
		remove(outputFileName.c_str());
		delete output;
		return;
	}
	
	headerString << "<?xml version=\"1.0\"?>" << endl;
	headerString << "<testsuite name=\""<< group << "\" failures=\""<< nbFailures << "\" success=\""<< nbSuccess << "\" errors=\""<< nbErrors << "\" skipped=\""<< nbSkipped <<  "\" time=\"" << totalTime << "\">" << endl;

	(*output) << headerString.str();
	(*output) << mainContentString.str();
	(*output) << "</testsuite>";
	CHECK(output->close());
	delete output;
	
}

void JobManager::pushJobsIntoFiles() {

	vector<FileManager*> vFiles = config->job().getJobsFiles();
	for(vector<FileManager*>::iterator it = vFiles.begin(); it != vFiles.end(); it++){
		pushInOneFile((*it));
	}
}

Job* JobManager::resolveADep ( std::string pattern ) {
	for(size_t i = 0; i < nbLists; i++){
		for(list<Job*>::iterator it = jobsList[i].begin(); it != jobsList[i].end(); it++){
			if ((*it)->getName() == pattern)
				return (*it);
		}
	}
	return NULL;
}

void JobManager::resolveAJobDeps ( Job* current ) {
	Job* dep;
	string* depName = NULL;
	
	assert(current != NULL);
	for(int i=0; i < current->getNbDeps(); i++){
		depName = current->getDepsNames().at(i);
		dep = resolveADep(*depName);
		if(dep != NULL)
			current->addDependency(dep);
		else {
			printWarning("Dependency \""+*depName+"\" not found !!!");
			current->updateStatus(INVALID_DEPS);
		}
	}
}

void JobManager::resolveJobsDeps() {
	for(size_t i = 0; i < nbLists; i++){
		for(list<Job*>::iterator it = jobsList[i].begin(); it != jobsList[i].end(); it++){
			resolveAJobDeps(*it);
		}
	}
}
size_t JobManager::getNbRemainingJobs() const
{
	return nbRemainingJobs;
}

void JobManager::update(int val) {
	nbRemainingJobs += val;
}

std::list< Job* >* JobManager::jobsLists() const
{
	assert(jobsList != NULL);
	return jobsList;
}

size_t JobManager::getNbLists() const
{
	return nbLists;
}
