/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef JOBMANAGER_H
#define JOBMANAGER_H

#include "utils.h"
#include "Job.h"
#include "Configuration.h"
#include "FilterBox.h"
#include "XMLJobParser.h"

/// Main class gathering all jobs for the current validation
/**
 * A Job manager is the main entity handling jobs (reading, writing results, etc...)
 * jobs are gathered according to their required resources number. There are one lists
 * for each type of job size. Thus, if the biggest runnable job requires 10 resources,
 * We will have 10 lists, for each requiremeents possibility.
 */
class JobManager {
private:
	/************** MEMBERS **************/
	std::list<Job*>* jobsList;          ///< The job lists array (a list per requiremeent possibility
	size_t nbLists;                     ///< the number of jobs list
	std::list<Job*> executedJobsList;   ///< the executed jobs list (after execution, before pushing)
	size_t nbRemainingJobs;             ///< number of jobs which still are in job Manager (at any t moment)
	size_t nbJobs;                      ///< total number of jobs parsed from files
	Configuration* config;              ///< current global configuration
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	void resolveAJobDeps( Job* current );    ///< used by jobManager to match job deps (as string) to real jobs (as pointer)
	Job* resolveADep( std::string pattern ); ///< Looking for the matching deps stored into the manager
	void pushInOneFile(FileManager* file);   ///< used by JobManager to write down results info output xml files
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit JobManager();
	///Create a JobManager
	/**
	 * \param[in] config current global configuration
	 */
	JobManager( Configuration* config );
	/// add current job to the manager
	/**
	 * When job is added, the manager looks for the good list where job will be pushed.
	 * \param[in] current current job to add to the list
	 */
	void addJob(Job* current);
	/// add current job to the executed jobs lists
	/**
	 * After jobs executed, they are pushed into an another and waiting for writing down
	 * \param[in] current the job to push into executed list
	 */
	void addExecutedJob(Job* current);
	/// main function to resolve jobs dependences
	/**
	 * Once all jobs are parsed from files, jobs deps are resolved and deps names list are
	 * replaced by a jobs pointers list, in order to access more quickly to the job's deps
	 */
	void resolveJobsDeps();
	///update number or remaining jobs ( adding val to class member)
	/**
	 * When jobs are added or removed from manager, an increment or decrement is realized
	 * on remaining jobs number
	 * \param[in] val number of jobs to add to remaining jobs
	 */
	void update(int val);
	///clean invalid jobs in manager
	/**
	 * When jobs are invalid because some of theirs deps are failed (or whatever else which
	 * block job launching), this function clean up manager to remove all of these not runnable
	 * jobs.
	 */
	void cleanInvalidJobs();
	/// parse jobs from xml files
	/**
	 * read from xml files each job and add them to the manager. XML Files are validated with
	 * a XML scheem, in order to avoid bad tests insertion
	 * \see XMLParser
	 * \warning if enabled, this function uses OpenMP directives to improves XML parsing when
	 * there are a lot of files.
	 */
	void pullJobsFromFiles();
	/// when scheduler have ended, pushes all jobs results into files
	void pushJobsIntoFiles();
	///virtual destructor to unset the job Manager
	virtual ~JobManager();
	/****** CONST ******/
	///print a job manager on screen (debug)
	void display() const;
	/// check if job Manager is job-free (all jobs lists are empty)
	/**
	 * \return <b>True</b> if Job Manager is empty
	 * \return <b>False</b> otherwise
	 */
	bool isEmpty() const;
	/// check if list at given indic is empty
	/**
	 * check if one list is empty. May be not used in our scheduler because we use vector
	 * iteration and the iterator stop is the vector is empty.
	 * Essentially used to debug
	 * \param[in] indice the list indice to check
	 * \return <b>True</b> if current list is empty
	 * \return <b>False</b> otherwise
	 */
	bool currentListIsEmpty(int indice) const;
	/// get the list with the best indice where there still are jobs
	/**
	 * This function returns the greatest jobs lists indice lower than current
	 * number of available resources
	 * \param[in] nbResources current number of availables resources
	 * \return the indice where there still are jobs
	 */
	int getHigherValidList(size_t nbResources) const;
	/// get the number of remaining jobs in the job Manager
	/**
	 * \return the number of remaining jobs
	 */
	size_t getNbRemainingJobs() const;
	/// get the total number of jobs stored into the job manager
	/**
	 * \return the total number of jobs
	 */
	size_t getNbJobs() const;
	/// get the jobs lists array
	/**
	 * Allow to get the jobs list array
	 * \warning this function allows you to update the jobsLists and eventually squash the whole job manager structure. Be careful with it. Nevertheless, the attribute pointer cannot be changed (no raalloation, etc...)
	 * \return the jobs lists array
	 */
	std::list<Job*>* jobsLists() const;
	/// get number of lists
	size_t getNbLists() const;
};

#endif // JOBMANAGER_H

