/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef JOBRESULT_H
#define JOBRESULT_H

#include "utils.h"
#include "FileManager.h"

///Stores results and informations about running for a given job
/**
 * When a job comes back from running, a JobResult is created and contains all
 * informations about the run. Some token are inserted to ensure data coherency, 
 * like hashes and magik numbers.
 */
class JobResult {
private:
	/************** MEMBERS **************/
	/// contains all informations about the run (except output)
	class JobResultHeader{
	public:
		size_t id;             ///< used to retrieve job after running
		size_t hashedName;     ///< the hashed job name
		size_t hashedData;     ///< the hashed output
		size_t sizeData;       ///< number of bytes in output
		double executionTime;  ///< effective execution time
		int finalReturnCode;   ///< effective return code
		double startingTime;   ///< timestamp when job started
		unsigned int magik;    ///< magik number (data coherency)
	} header;                          ///< header with global information about run
	std::string flow;                  ///< output
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	JobResult();
	///Fill header object with main information about run
	/**
	 * All this information are given by application and allow to generate
	 * a job result which could be directly written (binary) into file
	 * \param[in] jobName the job name
	 * \param[in] jobId the job id
	 * \param[in] rc the effective return code
	 * \param[in] time the execution time
	 * \param[in] begin the starting timestamp
	 */
	void fillHeader(std::string jobName, size_t jobId, int rc, double time, double begin);
	/// insert data flow info the jobresult
	/**
	 * Once header have been set, the job result is set with job output
	 * \param[in] data output to insert
	 */
	void insertData(std::string data);
	/// write the jobResult in the given file
	/**
	 * The job result will be written in two steps:
	 * <ol>
	 *  <li> write the header</li>
	 *  <li> write the flow</li>
	 * </ol>
	 * All writes are in binary mode
	 * \param[in] file the output file where results will be pushed
	 */
	void write(FileManager* file);
	/// read the jobResult from the given file
	/**
	 * Fill the current jobResult with data from given binary file
	 * The reading is done in two steps:
	 * <ol>
	 *  <li> read the header</li>
	 *  <li> read the flow</li>
	 * </ol>
	 * \param[in] file the input file
	 * \return <b>True</b> if read is succeeded
	 * \return <b>False</b> otherwise
	 */
	bool read(FileManager* file);
	///virtual destructor to unset a jobResult
	virtual ~JobResult();
	
	/****** CONST ******/
	/// get hashed name from header
	/**
	 * \return the hashed name
	 */
	size_t getHashName() const;
	/// get effective return code
	/**
	 * \return return code stored in header
	 */
	int getFinalRC() const;
	/// get magik number
	/**
	 * \return a value that should be DEFAULT_MAGIK_NUMBER
	 */
	unsigned int getMagik() const;
	/// print a jobresult on screen
	void display() const;
	/// check if jobresult read from file are valid
	/**
	 * Two fields are checked:
	 * <ol>
	 *  <li> Controls that hashed flow from file matches with value stored in header</li>
	 *  <li> check if magik number is equal to DEFAULT_MAGIK_NUMBER</li>
	 * </ol>
	 */	 
	void checksum() const;
	/// get job time
	/**
	 * \return the time (rounded to 2 numbers)
	 */
	double getTime() const;
	///get job start timestamp
	/**
	 * \return the starting time (rounded to 2 numbers)
	 */
	double getStartTime() const;
	/// get job result data
	/**
	 * \return a string containing job output
	 */
	std::string getData() const;
	/// get hashed data
	/**
	 * \return a int representing the hash
	 */
	size_t getHashData() const;
	/// get data number of chars
	/**
	 * \return the number of chars
	 */
	size_t getSizeData() const;
	/// get the job id
	/**
	 * In master, used to match the launched job with its result
	 * \return the job id
	 */
	size_t getId() const;
};

#endif // JOBRESULT_H
