/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#ifndef RUNNER_H
#define RUNNER_H
#include "Worker.h"
#include "JobManager.h"
#include "PolicyDefault.h"
#include "PolicySchedRes.h"
#include "PolicySchedTime.h"
#include "PolicyCustom.h"

///Represents directives to use to follow scheduling policy
/**
 * A Runner is a class which allow to deal with jobs and resources.
 * It's used as template for derived class RunnerMaster and RunnerSlave
 */
class Runner {
protected:
	/************** MEMBERS **************/
	JobManager* jobMan;     ///< referent job manager
	Configuration* config;  ///< current global configuration
	Policy* policy;         ///< policy to apply to handle jobs
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	//standard constructor
	explicit Runner();
	/// constructor which instantiate a Runner
	/**
	 * \param[in] job the referent job manager
	 * \param[in] config gloal configuration
	 */
	Runner(JobManager* job, Configuration* config);
	///< calls parsing files routines for xml files
	/**
	 * This function do some work about uploading jobs from xml files.
	 * Since work is the same in master and slave, the implementation
	 * is provided in the Runner
	 */
	virtual void pullJobsFromFiles();
	///< calls writing routines for jobs and pushing down into xml files
	/**
	 * This function writes executed jobs into their own JUnit xml Files
	 * The slave doesn't call this function (empty routine) because results
	 * are pushed from workers themselves. The master uses this call to
	 * write jobs in JUnit files
	 */
	virtual void pushJobsIntoFiles();
	/// main routine to fill a worker with jobs
	/**
	 * This routine do some pre and post actions about worker filling. 
	 * The main way to do is implemented by referent policy. This function
	 * is called only for fill "tests", i.e. jobs considered as "compilation"
	 * will have been executed before that.
	 * \param[out] cur the current corker to fill
	 * \param[in] nbResources availables resources number
	 */
	virtual void fillWorker ( Worker* cur, size_t nbResources);
	/// wait for next ended worker
	/**
	 * Do a wait call, retrieve the worker in the hash table and pull results
	 * from binary file
	 * \return A worker pointer on referent object
	 * \return <b>NULL</b> if worker not found
	 */
	virtual Worker* waitNextWorker();
	/// launch the worker and register it in the database
	/**
	 * Starts the worker and call this application in slave mode and according
	 * to launching options (batch managers, options...), and then saves it in
	 * hash table to find it out quickly later
	 * \param[in] cur the worker to launch
	 */
	virtual void launchWorker ( Worker* cur );
	/// do some work before workers creation
	/**
	 * This function is called each time a new worker is retrieved
	 */
	virtual void preActions();
	///< do some worker after worker retrieving
	/**
	 * In Master, this function dispatches jobs: if they've been 
	 * executed, there are remove from job Manager. Otherwise, they
	 * are rescheduled in the job manager
	 * \param[in] cur the worker to deal with
	 */
	virtual void postActions(Worker *cur);
	///< Specific function to launch compilation jobs first
	/**
	 * This function is not implemented in slave mode. Only master have
	 * implemented routine.
	 */
	virtual void startCompilation();
	/// virtual destructor to unset a Runner
	virtual ~Runner();
	
};

#endif // RUNNER_H

