/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#include "RunnerMaster.h"
using namespace std;

static HashTable *globalHashTable;
static JobManager*globalJobManager;

RunnerMaster::RunnerMaster() {}

RunnerMaster::RunnerMaster(JobManager* jobMan, Configuration* config) : Runner(jobMan, config), firstRound(true)
{
	struct sigaction sig1, sig2;
	
	globalJobManager = jobMan;
	globalHashTable = &tabWorkers;
	
	sig1.sa_flags=SA_SIGINFO;
	sig1.sa_sigaction=&RunnerMaster::handlerSignal;
	sigemptyset(&sig1.sa_mask);
	sigaddset(&sig1.sa_mask, SIGINT);
	if(sigaction(SIGINT, &sig1, NULL) < 0) printWarning("Unable to Handle SIGINT signal\n");
	sig2.sa_flags=SA_SIGINFO;
	sig2.sa_sigaction=&RunnerMaster::handlerSignal;
	sigemptyset(&sig2.sa_mask);
	sigaddset(&sig2.sa_mask, SIGALRM);
	if(sigaction(SIGALRM, &sig2, NULL) < 0) printWarning("Unable to Handle SIGALARM signal\n");
}

void RunnerMaster::fillWorker ( Worker* cur, size_t nbResources) {
	
	Runner::fillWorker ( cur , nbResources);
	size_t i = jobMan->getHigherValidList(nbResources);
	policy->fillingAlgo(cur, i, nbResources);
	jobMan->update((-1)*(int)cur->size());
}
void RunnerMaster::launchWorker ( Worker* cur ) {
	char * tab_args[NB_PARAMETERS_COMMAND+1];
	ostringstream uniqFlux;
	string in, out;
	pid_t desc;
	int cpt = 0;

	Runner::launchWorker ( cur );	
	uniqFlux << *config->system().getBuildDirectory() << "/testFile-" << setw(5) << setfill('0') << Worker::numWorker << ".in"; 
	in = uniqFlux.str();
	out = uniqFlux.str() + ".out";
	cur->setIOFiles(new FileManager(&in, OUTPUT_ACCESS),new FileManager(&out, INPUT_ACCESS_BIN));
	cur->pushJobsInputFile(jobMan->getNbRemainingJobs());

	//then, fork() + add an entry in hash tab
	desc = fork();
	if(desc == 0){
		for(unsigned int i=0; i<=NB_PARAMETERS_COMMAND; i++) tab_args[i] = NULL;
		// constructing launcher script to call before slave
		if(firstRound && config->system().getCompilationJobsLauncherCommand() != NULL) {// if compilation
			tab_args[cpt++] =  const_cast<char*>(config->system().getCompilationJobsLauncherCommand()->c_str());
		}
		else if(!firstRound && config->system().getJobsLauncherCommand() != NULL) { // for jobs
			tab_args[cpt++] = const_cast<char*>(config->system().getJobsLauncherCommand()->c_str());
		}
		// main minimal options to restart a slave
		tab_args[cpt++] = const_cast<char*>(config->getExeName().c_str());
		tab_args[cpt++] = (char*)"--slave";
		tab_args[cpt++] = const_cast<char*>(in.c_str());
		
		uniqFlux.str(""); uniqFlux << "--autokill=" << config->system().getMaxSlaveTime();
		tab_args[cpt++] = const_cast<char*>((new string(uniqFlux.str()))->c_str());
		
		uniqFlux.str(""); uniqFlux << "--nb-resources=" << cur->getNbResources(); 
		tab_args[cpt++] = const_cast<char*>((new string(uniqFlux.str()))->c_str());
		
		uniqFlux.str(""); uniqFlux << "--verbosity=" << config->job().getVerbosity();
		tab_args[cpt++] = const_cast<char*>((new string(uniqFlux.str()))->c_str());
		
		uniqFlux.str(""); uniqFlux << "--size-flow=" << config->system().getFlowMaxBytes();
		tab_args[cpt++] = const_cast<char*>((new string(uniqFlux.str()))->c_str());
		
		if(*config->system().getOutputDirectory() != "")
			tab_args[cpt++] = const_cast<char*>((new string("--output="+*config->system().getOutputDirectory()))->c_str()); 

		// conditional options
		if(config->job().isFake())
			tab_args[cpt++] = (char*)"--fake";
// 		if(config->job().getMaxTestTime() != JobConfiguration::DEFAULT_MAX_TEST_TIME)
		execvp(tab_args[0], tab_args);
		printError("A worker couln't be started ! ABORT\n");
	}
	else{
		tabWorkers.add(desc, cur);
	}
}
void RunnerMaster::postActions(Worker* cur) {
	Runner::postActions(cur);
	for(list<Job*>::iterator it = cur->begin(); it != cur->end();it++){
		if((*it)->getStatus() == NOT_RUN) {
			jobMan->addJob(*it);
			jobMan->update(1);
		}
		else{
			jobMan->addExecutedJob(*it);
		}
	}
	
	cur->unsetIOFiles();
}
void RunnerMaster::preActions() {
	Runner::preActions();
	jobMan->cleanInvalidJobs();
}
Worker* RunnerMaster::waitNextWorker() {
	pid_t pid;
	Worker* cur = NULL;
	int ret;

	pid = wait(&ret);
	if(pid == 130)
		handlerSignal(SIGINT, NULL, NULL);
	
	cur = tabWorkers.remove(pid);
	cur->pullJobsOutputFile();
	if(WIFEXITED(ret) && WEXITSTATUS(ret) == 255)
		handlerSignal(SIGINT, NULL, NULL);

	return cur;
}
void RunnerMaster::pushJobsIntoFiles()
{
	jobMan->pushJobsIntoFiles();
}

void RunnerMaster::pullJobsFromFiles()
{
    Runner::pullJobsFromFiles();
    if(!jobMan->isEmpty())
	printHeader();
}

void RunnerMaster::handlerSignal(int sig, siginfo_t * siginfo, void *context)
{
	UNUSED(siginfo);
	UNUSED(context);
	
	if(sig == SIGALRM)
		printInfo("* Interrupted by Autokill");
	else
		printInfo("* Interrupted by User");
	
	printInfo("* Interrupt child processes...");
	globalHashTable->stopAll();
	printInfo("* Saving current Data....");
	globalJobManager->pushJobsIntoFiles();
	printInfo("[ENDED]");
	exit(1);
}

void RunnerMaster::startCompilation()
{
	Worker cur;
	bool toTake = false;
	std::list<Job*>* currentJM = jobMan->jobsLists();
	int i = jobMan->getNbLists() - 1;
	
	if(jobMan->isEmpty()) return;
	
	Runner::startCompilation();
	
	do{
		for(list<Job*>::iterator it = currentJM[i].begin(); it != currentJM[i].end();){
			toTake = false;
			for(vector<JobConstraint*>::const_iterator cit = (*it)->getConstraints().begin(); cit != (*it)->getConstraints().end(); cit++){
				if((*cit)->getName() == JobConstraint::COMPILATION_TAG){
					toTake = true;
					break;
				}
			}
			if(toTake){
				cur.add(*it);
				jobMan->update(-1);
				it = currentJM[i].erase(it);
			}
			else{
				it++;
			}
		}
	i--;
	}while (i >=0);
	if(cur.size() > 0){
		cur.setNbRequiredResources(config->system().getNbMaxResources());
		launchWorker(&cur);
		waitNextWorker();
		postActions(&cur);
	}
	firstRound = false;
}
