/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#include "RunnerSlave.h"
using namespace std;

RunnerSlave::~RunnerSlave(){
	CHECK(outputFile->close());
#ifdef ENABLE_TRACE
	CHECK(traceFile->close());
#endif
	if(nextWorker != NULL) nextWorker->unsetIOFiles();
	pthread_mutex_destroy(&waitOnBusySocket);
	pthread_mutex_destroy(&waitOnEmptySocket);
}

void RunnerSlave::fillWorker ( Worker * cur, size_t nbResources ) {
	int id = 0;
	
	assert(cur != NULL);
	Runner::fillWorker ( cur, nbResources);
	
	id = jobMan->getHigherValidList(nbResources);
	assert(id >= 0);
	cur->add(jobMan->jobsLists()[id].front());
	cur->setNbRequiredResources(jobMan->jobsLists()[id].front()->getNbResources());
	jobMan->jobsLists()[id].pop_front();
}

void RunnerSlave::launchWorker ( Worker* cur ) {
	pthread_t thread;
	int err;
	ThreadItem* item = new ThreadItem;
	
	cur->setIOFiles(NULL, outputFile, traceFile);
	
	item->worker = cur;
	item->slave = this;
	Runner::launchWorker ( cur );

	if(pthread_create(&thread, NULL, launchWorkerThread, (void*)item))
		pthread_exit((void*)&err);
}

void RunnerSlave::postActions(Worker* cur) {
	Runner::postActions(cur);
	cur->pushJobsOutputFile();
}

void RunnerSlave::preActions() {
	Runner::preActions();
}

Worker* RunnerSlave::waitNextWorker() {
	Worker *cur = NULL;

	pthread_mutex_lock(&waitOnEmptySocket);
	//push next worker to dump
	cur = nextWorker;
	//unlock master to read the next worker
	pthread_mutex_unlock(&waitOnBusySocket);
		
	assert(cur != NULL);
	return cur;
}

RunnerSlave::RunnerSlave(JobManager* jobMan, Configuration* config): Runner(jobMan, config)
{
	std::string out = config->job().getJobsFiles().front()->toString() + ".out", trace;
	ostringstream flux;
	struct sigaction sig1, sig2;
	
	sig1.sa_flags=SA_SIGINFO;
	sig1.sa_sigaction=&RunnerSlave::handlerSignal;
	sigemptyset(&sig1.sa_mask);
	sigaddset(&sig1.sa_mask, SIGINT);
	if(sigaction(SIGINT, &sig1, NULL) < 0) printWarning("Unable to Handle SIGINT signal\n");
	sig2.sa_flags=SA_SIGINFO;
	sig2.sa_sigaction=&RunnerSlave::handlerSignal;
	sigemptyset(&sig2.sa_mask);
	sigaddset(&sig2.sa_mask, SIGALRM);
	if(sigaction(SIGALRM, &sig2, NULL) < 0) printWarning("Unable to Handle SIGALARM signal\n");	
	
	outputFile = new FileManager(&out, OUTPUT_ACCESS_BIN);
	traceFile = NULL;
	CHECK(outputFile->open());
	pthread_mutex_init(&waitOnBusySocket, NULL);
	pthread_mutex_init(&waitOnEmptySocket, NULL);
	nextWorker = NULL;
	pthread_mutex_lock(&waitOnEmptySocket);
	
	srand(time(NULL) * getpid());
	
#ifdef ENABLE_TRACE
	trace = (*config->system().getOutputDirectory() != "") ? *config->system().getOutputDirectory()+ "/" +config->job().getJobsFiles().front()->getName(): config->job().getJobsFiles().front()->toString();
	trace += ".trace";
	traceFile = new FileManager(&trace, OUTPUT_ACCESS);
	CHECK(traceFile->open());
	flux << "#nbJobs:" << config->system().getNbMaxResources() << ":" << fixed << setprecision(2) << getCurrentDate() << endl;
	(*traceFile) << flux.str();
#endif

}

void* RunnerSlave::launchWorkerThread(void* arg)
{
	ThreadItem* item = (ThreadItem*)arg;

	//Start the launch for the job
	for(list<Job*>::iterator it = item->worker->begin(); it != item->worker->end(); it++)
		item->slave->launchWorkerThreadStart(*it);
	
	//waiting for master availability
	pthread_mutex_lock(&item->slave->waitOnBusySocket);
	//push next worker to dump
	item->slave->nextWorker = item->worker;
	//unlock master to read the next worker
	pthread_mutex_unlock(&item->slave->waitOnEmptySocket);
	
	return NULL;
}

void RunnerSlave::launchWorkerThreadStart(Job* job)
{
	int temp = -1, finalRC, rnd;
	FILE* fd;
	DataFlow flow(config->system().getFlowMaxBytes());
	double start = 0.0, end = 0.0;
	struct timespec ts;
	string instruction;
	
	if(config->job().isFake()){
		start = getCurrentDate();
		rnd = ((rand()%(MAX_RANDOM-MIN_RANDOM)+MIN_RANDOM))*VAR_COEFF;
		ts.tv_sec = (int)rnd/NB_MS_IN_SEC;
		ts.tv_nsec = ((long)rnd-(ts.tv_sec*NB_MS_IN_SEC))*NB_NS_IN_MS;
		nanosleep(&ts, NULL);
		end = getCurrentDate();
		finalRC = job->getExpectedReturn();
		flow.fillSpecificMessage("Fake execution");
	}
	else{
		instruction = job->getCommand() + " 2>&1";
		start = getCurrentDate();
		// read + close on exec flag
		fd = popen(instruction.c_str(), "re");
		flow.fillFrom(fd);	
		temp = pclose(fd);	
		end = getCurrentDate();
		finalRC = (WIFEXITED(temp)) ? WEXITSTATUS(temp): -1 ;
	}
	
	job->addResult(finalRC, end - start, start , flow.getContent());
	
	if (finalRC == job->getExpectedReturn())
		printLine(PREFIX_PASS, "%6d/%-6d (%6d) %8.2f   %s\n",job->getId(), jobMan->getNbJobs(), jobMan->getNbRemainingJobs(), job->getResult().getTime(), job->getName().c_str());
	else 
		printLine(PREFIX_FAIL, "%6d/%-6d (%6d) %8.2f   %s\n",job->getId(), jobMan->getNbJobs(), jobMan->getNbRemainingJobs(), job->getResult().getTime(), job->getName().c_str());
	
	if((config->job().getVerbosity() == VERBOSE_ONLY_ERROR &&  finalRC != job->getExpectedReturn()) || config->job().getVerbosity() >= VERBOSE_ALL){
		cout << "COMMAND = " << job->getCommand() << endl;
		cout << flow.getContent() << endl;
	}
}

void RunnerSlave::handlerSignal(int sig, siginfo_t * siginfo, void *context)
{
	assert(siginfo != NULL);
	assert(context != NULL);
	UNUSED(sig);
	UNUSED(siginfo);
	UNUSED(context);
	//notify the master
	exit(130);
}
