/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#ifndef WORKER_H
#define WORKER_H
#include "FileManager.h"
#include "Job.h"

/// represents a group of jobs to launch
/**
 * A worker is created in both master and slave. Its aim is to be filled with
 * jobs and follows them during their execution. We distinguish two differents
 * behaviours:
 * <ul>
 * <li>
 * MASTER: a worker is considered as a group of jobs transmitted to the slave for
 * execution. The worker writes down communication information between instances
 * himself. A pointer is kept on the worker until slave terminated. Then, worker pulls
 * data from file and updates its jobs. At the end, these jobs are pushed in job manager
 * and the worker is deleted
 * </li>
 * <li>
 * SLAVE: A worker is just the entity who follows the job from it's starting selection until
 * its data writing down into binary file. A job which is pushed into a worker won't be
 * put again in the job manager. The worker handles the job's end of life.
 * </li>
 * </ul>
 */
class Worker {
private:
	/************** MEMBERS **************/
	size_t nbRequiredResources;   ///< number of required resources for this launch
	FileManager* inputFile;       ///< where data will be written (and read by slave)
	FileManager* outputFile;      ///< where data will be read (and write by slave)
	FileManager* traceFile;       ///< where log will be pushed (jsloc)
	std::list<Job*> jobsList;     ///< list of jobs to execute
	
public:
	/************** STATICS **************/
	/**** NON-CONST ****/
	static unsigned int numWorker;///< worker id incrementation (used to number temp files)
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit Worker();
	/// set I/O files for the worker
	/**
	 * Inverted behaviour between master and slave.
	 * \param[in] input the input file to push(pull) data
	 * \param[in] output the output file to pull(push) data
	 * \param[in] trace eventually, a trace file where log will be pushed
	 */
	void setIOFiles(FileManager* input, FileManager* output, FileManager* trace = NULL);
	/// destroy FileManagers
	/**
	 * Specific function to avoid leak memory --> unset its own file Manager
	 */
	void unsetIOFiles();
	/// configure number of required resources once worker if filled
	/**
	 * \param[in] val number of resources to reserve
	 */
	void setNbRequiredResources( size_t val );
	/// add current job to the worker
	/**
	 * \param[in] job the job to add
	 */
	void add(Job* job);
	/// push the whole job in worker into temporary files (start with slave)
	/**
	 * Reserved to MASTER
	 * \param[in] nbRemain number of remaining jobs when the worker is launched
	 */
	void pushJobsInputFile(int nbRemain);
	/// push the whole jobs results into binary file (slave end)
	/**
	 * Reserved to SLAVE
	 */
	void pushJobsOutputFile();
	/// read results from output file
	/**
	 * Reserved to MASTER
	 */
	void pullJobsOutputFile();
	/// Get the first job in the worker list
	/**
	 * \return an iterator on the first job item
	 */
	std::list<Job*>::iterator begin();
	/// Get the last job in the worker list
	/**
	 * \return an iterator on the last job item
	 */
	std::list<Job*>::iterator end();
	/// virtual destructor to unset a worker
	virtual ~Worker();
	
	/****** CONST ******/
	/// get number of required resources
	/**
	 * \return the number of resources registerd for the current worker
	 */
	size_t getNbResources() const;
	/// get the input file set for the worker
	/**
	 * \return the configured input file
	 */
	FileManager* getInputFile() const;
	/// get the output file set for the worker
	/**
	 * \return the configured output file
	 */
	FileManager* getOutputFile() const;
	/// display a worker on screen
	void display() const;
	/// get the number of jobs in the worker
	/**
	 * \return the numbe of jobs
	 */
	size_t size() const;
	
};

#endif // WORKER_H
