/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#ifndef XMLCONFIGPARSER_H
#define XMLCONFIGPARSER_H

#include "XMLParser.h"

/// class handling configuration file
/**
 * Instead of using arguments line, options can be loaded with a configuration
 * file. This class extracts data from the file and fill a given configuration
 */
class XMLConfigParser : private XMLParser {
private:
	/************** MEMBERS **************/
	xmlNodePtr jobNode;    ///< a pointer on \<job/\> in configuration file
	xmlNodePtr systemNode; ///< a pointer on \<system/\> in configuration file
	
public:
	static const bool JOB_CONF = true;  ///< looking for job options
	static const bool SYS_CONF = false; ///< looking for system options
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// cosntruct a parser with the given file name
	/**
	 * This function validates the given file with pre-registered xml schema
	 * and set class members.
	 * \param[in] fileName configuration file
	 */
	XMLConfigParser(std::string fileName);
	/// validates current config file with xml schema
	void checkConfig();
	/// load a node content as a integer value
	/**
	 * This function loads the content of nodeName node, located is the subtree
	 * belonging to parent (job or system ones). Then, the value is casted into
	 * integer and assigned to val
	 * \param[out] val where to store value
	 * \param[in] parent a bool to select job or system parent node
	 * \param[in] nodeName node name where data will be extracted.
	 */
	void loadInt(int& val, const bool parent, const char nodeName[]) const;
	/// load a node content as a string value
	/**
	 * This function loads the content of nodeName node, located is the subtree
	 * belonging to parent (job or system ones). Then, the value is casted into
	 * string and assigned to val
	 * \param[out] val where to store value
	 * \param[in] parent a bool to select job or system parent node
	 * \param[in] nodeName node name where data will be extracted.
	 */
	void loadStr(std::string& val, const bool parent, const char nodeName[]) const;
	/// fill a list with data stored in given node
	/**
	 * This function identifies a given node. Then, it extracts all \<item/\> nodes
	 * and, for each one, add a new entry in given list. Each node is considered as
	 * file name and thus, a FileManager is directly created as input file
	 * \param[out] list where to store value
	 * \param[in] parent a bool to select job or system parent node
	 * \param[in] globalNodeName global node name gathering \<item/\> nodes.
	 */
	void loadList(std::vector < FileManager * >&list, const bool parent, const char globalNodeName[]) const;
	/// load a node content as a boolean value
	/**
	 * This function loads the content of nodeName node, located is the subtree
	 * belonging to parent (job or system ones). Then, the value is casted into
	 * boolean and assigned to val
	 * \param[out] val where to store value
	 * \param[in] parent a bool to select job or system parent node
	 * \param[in] nodeName node name where data will be extracted.
	 */
	void loadBool(bool& val, const bool parent, const char nodeName[]) const;
	/// virtual destructor to unset a Config parser
	~XMLConfigParser();
};

#endif // XMLCONFIGPARSER_H
