/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/

#include "XMLJobParser.h"
using namespace std;


XMLJobParser::XMLJobParser() {

}

XMLJobParser::XMLJobParser(FileManager* name, FilterBox* const filtering) {
	nameFile = name;
	filter = filtering;
	
	document = xmlParseFile(name->toString().c_str());
	/* check if file exists */
	if (document == NULL) {
		printError("XML file '"+name->toString()+"' not found !\n");
		abort();
	}
}

XMLJobParser::~XMLJobParser() {
	xmlFreeDoc(document);
}

list<Job*>* XMLJobParser::parseFromFile(size_t * property) {
	XMLValidator validator;

	if(validator.validate(document, XMLValidator::xsdJobsvalidation))
		return parseJobsSuite(property);
	else {
		printWarning("XML file '"+nameFile->toString()+"' not valid !\n");
		return NULL;
	}
}

void XMLJobParser::parseDeps(std::vector<std::string*>& list, xmlNodePtr location) {
	xmlNodePtr child = NULL;
	string *dep;
	
	assert(list.empty() == true);
	
	checkNodeExists(location, (char*)"job");
	
	child = findChildNode(location, (char*)"deps");
	if(child == NULL) return;

	child = findChildNode(child, (char*)"dep");
	
	while(child != NULL){
		dep = new string(getNodeContent(child));
		list.push_back(dep);
		child = getNextSiblingNode(child);
	}
}

void XMLJobParser::parseConstraints ( std::vector< JobConstraint* >& list, xmlNodePtr location ) {
	xmlNodePtr child = NULL;
	string* constraint;
	
	assert(list.empty() == true);
	
	checkNodeExists(location, (char*)"job");

	
	child = findChildNode(location, (char*)"constraints");
	if(child == NULL) return;

	child = findChildNode(child, (char*)"constraint");
	
	while(child != NULL){
		constraint = new string(getNodeContent(child));
		
		list.push_back(new JobConstraint(*constraint));
		delete constraint;
		child = getNextSiblingNode(child);
	}
}

Job* XMLJobParser::parseJob(xmlNodePtr location) {
	string name, command, chain;
	size_t nbResources = 1;
	int rc = 0;
	double time = -1;
	bool directly_selected = false;
	vector<string*> vDeps;
	vector<JobConstraint*> vConstraints;
	
	checkNodeExists(location, (char*)"job");

	name = findChildNodeContent(location,(char*)"name");
	
	command = findChildNodeContent(location,(char*)"command");
	if ((chain = findChildNodeContent(location, (char*)"resources").c_str()) != "")
		nbResources = atoi(chain.c_str());
	if ((chain = findChildNodeContent(location, (char*)"rc").c_str()) != "")
		rc = atoi(chain.c_str());
	if ((chain = findChildNodeContent(location, (char*)"time").c_str()) != "")
		time = atof(chain.c_str());

	parseDeps(vDeps, location);
	parseConstraints(vConstraints, location);

	for(vector<JobConstraint*>::iterator it = vConstraints.begin(); it != vConstraints.end() && !directly_selected; it++){
		if((*it)->getName() == JobConstraint::COMPILATION_TAG)
			directly_selected=true;
	}

	if(!filter->accept(name, jobsGroup) && !directly_selected)
		return NULL;
	
	return new Job(name, command, jobsGroup, nbResources, rc, time , vDeps, vConstraints, nameFile);	
}

std::list<Job*>* XMLJobParser::parseJobsSuite(size_t *property) {
	xmlNodePtr root = xmlDocGetRootElement(document), cur;
	list<Job*>* group = NULL;
	Job* curJob = NULL;
	char * prop = NULL;
	
	group = new list<Job*>();
	checkNodeExists(root, (char*)"jobSuite");

	if((prop = (char*)xmlGetProp(root, (const xmlChar *)"package")) != NULL) {
		jobsGroup = prop;
		free(prop);
	} 
	else if((prop = (char*)xmlGetProp(root, (const xmlChar *)"nbRemain")) != NULL){
		(*property) = atoi(prop);
		free(prop);
	}
	
	cur = getNextChildNode(root);
	while(cur != NULL){
		curJob = parseJob(cur);
		if(curJob != NULL) group->push_back(curJob);
		cur = getNextSiblingNode(cur);
	}
	
	return group;
}
