/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#ifndef XMLJOBPARSER_H
#define XMLJOBPARSER_H

#include "Job.h"
#include "FilterBox.h"
#include "XMLParser.h"

/// Interface between XML Jobs files and the application.
/**
 * This class wrappes LibXML2 using, with routines helping to extract data from DOM tree
 * More specifically, XMLJobParser is destinated to load data from XML Job File. This include
 * a xml validation with a xml scheme defined in XMLValidator.
 */
class XMLJobParser : private XMLParser {
private:
	/************** MEMBERS **************/
	std::string jobsGroup; ///< the package attached to the loaded file
	FilterBox* filter;     ///< the whole filter used to decide whether a job is valid or not
	
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	/// start file reading
	/**
	 * After some preActions, the XMLParser::parseFromFile function starts file dumping. This function loads
	 * each \<job\>\</job\> and create the associated job object.
	 * \param[out] property For the slave only, the property is set with remaining jobs number in master
	 * \return A jobs list pointer
	 * \return <b>NULL</b> if there are no job to parse
	 */
	std::list< Job* >* parseJobsSuite(size_t* property);
	/// create one job from current reading
	/**
	 * For each \<job\>\</job\> encoutered, this function is called to extract data from file and return a 
	 * Job pointer
	 * \param[in] location the current "job" node in the file
	 * \return a job pointer on new created object
	 */
	Job* parseJob( xmlNodePtr location );
	/// parse the whole constraints for a given job
	/**
	 * This function load all constraints depicted in the file for the given job
	 * \param[out] list the constraints' list to fill
	 * \param[in] location the "constraint" node in the file for the current job
	 * 
	 */
	void parseConstraints( std::vector< JobConstraint* >& list, xmlNodePtr location );
	/// parse the whole dependences for a given job
	/**
	 * This function load all dependences depicted in the file for the given job
	 * \param[out] list the dependences' list to fill
	 * \param[in] location the "dependences" node in the file for the current job
	 */
	void parseDeps( std::vector< std::string* >& list, xmlNodePtr location );
	
public:
	/************* FUNCTIONS *************/
	/**** NON-CONST ****/
	///standard constructor
	explicit XMLJobParser();
	///constructor initializing the default XML Parser
	XMLJobParser( FileManager* name, FilterBox* const filtering );
	///dump data from file and generate a jobs list
	/**
	 * This function read data from file assigned by constructor and generate a pointer
	 * on a jobs list, with all valid jobs (i.e. the filtering is done here)
	 * \param[out] property For the slave only, the property is set with remaining jobs number in master
	 * \return A jobs list pointer
	 * \return <b>NULL</b> if there are no job to parse
	 */
	std::list< Job* >* parseFromFile(size_t* property);

	///virtual destructor to unset the parser
	virtual ~XMLJobParser();
};

#endif // XMLPARSER_H
