/****************************************************************************
 *                                                                          *
 * Tue Jul 22 13:28:10 CEST 2014                                            *
 * Copyright or (C) or Copr. Commissariat a l'Energie Atomique              *
 *                                                                          *
 * This file is part of JCHRONOSS                                           *
 *                                                                          *
 * JCHRONOSS is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Lesser General Public License as           *
 * published by the Free Software Foundation, either version 3 of the       *
 * License, or (at your option) any later version.                          *
 *                                                                          *
 * JCHRONOSS is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU Lesser General Public License for more details.                      *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public License *
 * along with JCHRONOSS.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                          *
 * Version : 1.0                                                            *
 * Author  : Julien Adam <julien.adam@cea.fr>                               *
 *                                                                          *
 ***************************************************************************/


#ifndef UTILS_H
#define UTILS_H

/************* INCLUSION *************/
/******* C LIBS ******/
#include <cstdio>
#include <libgen.h>
#include <cerrno>
#include <cstring>
#include <cstdlib>
#include <cassert>
#include <cstdarg>
/****** C++ LIBS *****/
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <iomanip>
#include <vector>
#include <list>
#include <functional>
/****** UNIX LIBS *****/
#include <getopt.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>

/*************** MACROS **************/
#define printError(chain)   do{ std::cerr<<COLOR_FAIL" ERROR        : "<< chain <<COLOR_NORM << endl; exit(1); } while(0)
#define printWarning(chain) do{ std::cerr<<COLOR_URUN" WARNING      : "<< chain <<COLOR_NORM << endl;          } while(0)
#define printInfo(chain)    do{ std::cout<<COLOR_INFO" INFO         : "<< chain <<COLOR_NORM << endl;          } while(0)
#define UNUSED(x) (void)(x)                  ///< define an unused paramater
#define NB_MS_IN_SEC (1000)                  ///< define the number of milliseconds in a second
#define NB_NS_IN_MS (1000000)                ///< define the number of nanoseconds in a millisecond
#define MAX_RANDOM 5000                      ///< define max random time in millisecond
#define MIN_RANDOM 0                         ///< define min random time
#define VAR_COEFF 0.5                        ///< define variance on random time
#define DEFAULT_MAGIK_NUMBER 42              ///< define magik number to ensure data coherency
#define DEFAULT_OUTPUT_FILENAME "/output-"   ///< define default prefix filename for output xml file
#define safeOpenRead(u, v)  do{ u = fopen(v, "r"); assert(u != NULL);} while(0)
#define safeOpenWrite(u, v) do{ u = fopen(v, "w"); assert(u != NULL);} while(0)
#define safeClose(u) do{fclose(u); u = NULL;}while(0)

/****** COLORS *****/
#ifdef ENABLE_COLOR
#define COLOR_INFO "\033[1;36m"
#define COLOR_PASS "\033[1;32m"
#define COLOR_FAIL "\033[1;31m"
#define COLOR_NRUN "\033[1;34m"
#define COLOR_URUN "\033[1;33m"
#define COLOR_DEBG "\033[1;36m"
#define COLOR_NORM "\033[0;0m"
#else
#define COLOR_INFO ""
#define COLOR_PASS ""
#define COLOR_FAIL ""
#define COLOR_NRUN ""
#define COLOR_URUN ""
#define COLOR_NORM ""
#define COLOR_DEBG ""
#endif

/****** PREFIX_COLOR *****/
#define PREFIX_INFO COLOR_INFO" INFO         : "
#define PREFIX_PASS COLOR_PASS" PASSED       : "
#define PREFIX_FAIL COLOR_FAIL" FAILED       : "
#define PREFIX_NRUN COLOR_NRUN" NOT RUN      : "
#define PREFIX_URUN COLOR_URUN" NOT RUNNABLE : "
#define PREFIX_DEBG COLOR_DEBG" DEBUG        : "

#ifdef NDEBUG
#define CHECK(u) u
#define printDebug(chain)
#else
#define CHECK(u) assert(u)
#define printDebug(chain) do{ std::cout<<COLOR_DEBG" DEBUG        : "<< chain <<COLOR_NORM << endl;          } while(0)

#endif

/****** COUT FORMATTING *****/
#define PADDED_HELP setw(40) << left
#define PADDED_OPT  setw(35) << left

/************* FUNCTIONS *************/
/// get current timestamp
/**
 * Using gettimeofday(), return number of seconds to
 * express current time
 */
double getCurrentDate();
/// print a line on screen according to prefix
/**
 * Prefix used to uniform printing method but differentiate
 * color format to apply to the output
 * \param[in] prefix the prefix to apply on the output
 * \param[in] format the formatted chain
 * \param[in] ... variable args list to print into formatted chain
 */
void printLine ( std::string prefix, std::string format, ... );
/// read with safety to avoid intempestive interruption
/**
 * When read from pipe, ensure that no interruption could block
 * output logging when the two processes communicates
 * \param[in] fd the pipe's file descriptor where read data
 * \param[in] buf the buffer where data will be read
 * \param[in] size the max size to read in once
 * \return effective number of read bytes
 */
int safeRead(FILE* fd, void* buf, int size);
/// remove from a chain, a defined prefix (used to parse options)
/**
 * \param[in] chain the chain containing the option + prefix
 * \param[in] prefix the prefix to remove
 * \return a string pointer on new option
 */
std::string* splitArgumentPrefix ( std::string chain, std::string prefix );
/// check if an option matches with a defined pattern
/**
 * checking the prefix.size() first characters in chain. if matches, then chain
 * is rattached to this option
 * \param[in] chain the chain to check
 * \param[in] prefix the option
 * \return <b>True</b> if option matches
 * \return <b>False</b> otherwise
 */
bool matchWith (std::string chain,std::string prefix );
/// print JCHRONOSS banner
void banner();
/// escape given string with specific characters patterns
/**
 * \param[in] src the string to escape
 */
void HTMLEncoding(std::string& src);
///print an header helping to identify jobs columns printing
void printHeader();
/// hash the given string with pseudo-unique value
size_t hash_fn(std::string toHash);
#endif
